#!/usr/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T10
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################

# USAGE:    set_openclbs.ksh <config_file>
#
# $Header: /net/klstor01/datavol/gateways/da/CVSROOT/modules/OPENCLBS/scripts/set_openclbs.ksh,v 1.3 2008/05/05 03:45:47 khalidm Exp $
#
# FUNCTION:   For each data collection specified in Openwave CLBS config_file,
#             this script will create the following:
#             data directories
#             log directories
#             icf configuration file
#             icf checkpoint file
#             DA monitor file
#             And one cron file in the ${FLEXDAHOME}/cfg directory
#

#
# Display usage message
#

usage() {
    echo "Usage: ${cmd} <config_file>"
    echo "     <config_file> is the Openwave CLBS configuration file"
    exit 1
}


################################################################################
# create_file -- Backup existing file before overwriting
################################################################################
create_file() {

    typeset fd=$1
    typeset file=$2
	action="Creating"

    if [ -f $file ]
    then
		test -d $SAVEDIR || mkdir -p $SAVEDIR
		cp $file $SAVEDIR
		action="Modifying"
    fi

    if [ $fd -gt 2 ]
    then
	# This results to an IO malfunction
	# fcmd="exec $fd>$file"
	# eval $cmd
	# This one works but not pretty!
	case $fd in
	    3) exec 3>$file;;
	    4) exec 4>$file;;
	    5) exec 5>$file;;
	    6) exec 6>$file;;
	    7) exec 7>$file;;
	    8) exec 8>$file;;
	    9) exec 9>$file;;
	esac
    fi

    echo "${cmd}: ${action} $(basename $file) file"
    echo; sleep 1
}


#
# Prints ICF common information
#

print_common_icf() {
    typeset fd=${1:-1}
    typeset title=${2:-"FlexDA ICF Configuration File"}
    typeset market=${3:-"goodmarket"}
    typeset netype=${4:-"netype"}
    typeset neid=${5:-"neid"}
    typeset dtype=${6:-"dtype"}

    print -u$fd "#"
    print -u$fd "# TITLE:   $title"
    print -u$fd "#"
    print -u$fd "# SITE:    $FLEXDASITE"
    print -u$fd "#"
    print -u$fd "# HOST:    $HOST"
    print -u$fd "#"
    print -u$fd "#"
    print -u$fd "# ICF_CHAT Variables:"
    print -u$fd "#"
    print -u$fd "#    ICFBIN        Location of ICF binaries."
    print -u$fd "#    ICFSCRIPT     Location of ICF scripts."
    print -u$fd "#    ICFTABLE      Filename of ICF checkpoint table."
    print -u$fd "#    ICFAPP        Filename of ICF application."
    print -u$fd "#"
    print -u$fd ""
    print -u$fd "set MARKET         \"$market\""
    print -u$fd "set NETYPE         \"$netype\""
    print -u$fd "set NEID           \"$neid\""
    print -u$fd "set DTYPE          \"$dtype\""
    print -u$fd 'set TARGET         "$NEID.$NETYPE"'
    print -u$fd ""
    print -u$fd 'set ICFBIN         "$env(FLEXDAHOME)/bin"'
    print -u$fd 'set ICFSCRIPT      "$env(FLEXDAHOME)/scripts"'
    print -u$fd 'set ICFTABLE       "$env(FLEXDAHOME)/log/$MARKET.$TARGET.chk"'
    case $dtype in
        clbscp|clbsup )     print -u$fd 'set ICFAPP         "$ICFSCRIPT/clbsdist.exp"';;
        *)                  print -u$fd 'set ICFAPP         "$ICFSCRIPT/$DTYPE.exp"';;
    esac
    print -u$fd ""
}


#
# Prints ICF's targethost/destination information
#

print_thost_icf() {
    typeset fd=${1:-1}
    typeset tip=${2:-tip}
    typeset tuser=${3:-tuser}
    typeset tpswd=${4:-tpswd}
    typeset tdir=${5:-tdir}

    print -u$fd "#"
    print -u$fd "# Target Prospect Server Variables"
    print -u$fd "#"
    print -u$fd "#    FLEXPMHOST    Address of Vallent Prospect server"
    print -u$fd "#    FLEXUSER      User account on Vallent Prospect server"
    print -u$fd "#    FLEXPSWD      User password on Vallent Prospect server"
    print -u$fd "#                  use single code to enclose password if password'"
    print -u$fd "#                  contains character such as $, for example 'a\$c'"
    print -u$fd "#    FLEXDIR       Prospect server dist destination directory"
    print -u$fd ""
    print -u$fd "set FLEXPMHOST     \"$tip\""
    print -u$fd "set FLEXUSER       \"$tuser\""
    print -u$fd "set FLEXPSWD       {$tpswd}"
    print -u$fd "set FLEXDIR        \"$tdir\""
    print -u$fd ""
}

#
# Prints CLBS icf
#

print_clbs_icf() {
    typeset fd=${1:-0}
    typeset tdir=${2:-tdir}
    typeset ip=${3:-ip}
    typeset user=${4:-user}
    typeset pswd=${5:-pswd}
    typeset id=${6:-id}
    typeset sdir=${7:-sdir}
    typeset scurcol=${8:-1}
    typeset scurdlvr=${9:-1}
    typeset lbstype=${10:-0}
    typeset ndays=${11:-3}
    typeset tz=${12:-GMT}

    print -u$fd "#"
    print -u$fd "# Application Variables:"
    print -u$fd "#"
    print -u$fd "#    SRCIP         Source IP address of the network element"
    print -u$fd "#    SRCID         Source ID of the network element."
    print -u$fd "#    SRCUSER       Login ID of the network element"
    print -u$fd "#    SRCPSWD       Login password of the netwok element."
    print -u$fd "#                  Use single code to enclose password if password"
    print -u$fd "#                  contains character such as $, for example 'a$c'"
    print -u$fd "#    SRCDIR        Remote source data directory"
    print -u$fd "#                  Use single code to enclose the path if the path"
    print -u$fd "#                  contains character such as $, for example 'a$c'"
    print -u$fd "#    SECURECOLLECT Whether to use SFTP/SSH to collect data"
    print -u$fd "#    SECUREDELIVER Whether to use secure SFTP for delivering data to Prospect"
    print -u$fd "#    LBSTYPE       Type of LBS Plane"
    print -u$fd "#    LOCALDIR      Local directory for the collected data"
    print -u$fd "#    NDAYS         Number of days to keep files in FlexDA server"
    print -u$fd "#    TZ            Unix Standard Time Zone For the Remote source server"
    print -u$fd "#\n"
    print -u$fd "set SRCIP          \"$ip\""
    print -u$fd "set SRCID          \"$id\""
    print -u$fd "set SRCUSER        \"$user\""
    print -u$fd "set SRCPSWD        {$pswd}"
    print -u$fd "set SRCDIR         {$sdir}"
    print -u$fd "set SECURECOLLECT  \"$scurcol\""
    print -u$fd "set SECUREDELIVER  \"$scurdlvr\""
    print -u$fd "set LBSTYPE        \"$lbstype\""
    print -u$fd 'set LOCALDIR       "$env(FLEXDAHOME)/data/$MARKET/$NEID/$NETYPE"'
    print -u$fd "set NDAYS          \"$ndays\""
    print -u$fd "set TZ             \"$tz\""
    print -u$fd ""
}

#

check_cfg_file() {
    typeset cfg=${1}
    typeset tmpcfg=/tmp/da.cfg.$$
    typeset tmplog=/tmp/da.err.$$

    if [ ! -f $cfg ]; then
        echo "${cmd}: Configuration file $cfg not found"
        exit 1
    fi

# Remove invalid data in the configuration file
    cat $cfg | sed 's/\\ *$/\\/' | sed 's/ *= */=/g' | sed 's/ *, */,/g' | \
        sed '/^$/d' > $tmpcfg
#cat $cfg | sed -e 's/\\ *$/\\/' -e 's/ *= */=/g' -e 's/ *, */,/g' \
#  -e '/^$/d' > $tmpcfg

# Source configuration file and check for error
. $tmpcfg 2>&1 2>/$tmplog

if [ -s $tmplog ]; then
    echo "${cmd}: Error reading configuration file: $cfg"
    exit 1;
fi

# Remove temporary files
rm -f $tmpcfg $tmplog


# Ensure that the correct number of attributes (na) have been specified
# for lhost thost Openwave CLBS
typeset na_lhost=1
typeset na_thost=4
typeset na_openclbs=1

if ( [ `expr ${#lhost[*]} % $na_lhost` != 0 ] || \
     [ `expr ${#thost[*]} % $na_thost` != 0 ] || \
     [ `expr ${#openclbs_type[*]} % $na_openclbs` != 0 ]); then
    echo "${cmd}: Incorrect number of arguments for lhost, thost, or openclbs"
    echo "${cmd}: check the arguments for lhost or thost in $1"
    exit 1
fi

typeset j=0
typeset i=0
while (($j < ${#openclbs_type[*]})); do
    # Get detailed information for each openclbs type
    set -A detail $(eval echo \${openclbs_${openclbs_type[$j]}[*]})

    case ${openclbs_type[$j]} in
        clbscp|clbsup ) na_detail=10;;
        *)  echo "${cmd}: Invalid option name ${openclbs_type[$j]}"; exit 1;;
    esac

    if ([ ${#detail[*]} == 0 ] || [ `expr ${#detail[*]} % $na_detail` != 0 ]); then
        echo "${cmd}: openclbs_${openclbs_type[$j]} not specified properly"
        ((j=j+1))
        continue
    fi

    # Get openclbs type detailed info
    typeset k=0
    while (($k < ${#detail[*]})); do
        unset d_market[$i] d_tdir[$i] d_srcip[$i] d_srcid[$i]
        unset d_srcuser[$i] d_srcpswd[$i] d_sdir[$i] d_stz[$i]
		unset d_scurcol[$i] d_scurdlvr[$i] d_lbstype[$i]
        unset d_mtopts[$i]

        case ${openclbs_type[$j]} in
            clbscp )
                d_market[$i]=${detail[$k]};   ((k=k+1))
                d_tdir[$i]=${detail[$k]};     ((k=k+1))
                d_srcip[$i]=${detail[$k]};    ((k=k+1))
                d_srcid[$i]=${detail[$k]};    ((k=k+1))
                d_srcuser[$i]=${detail[$k]};  ((k=k+1))
                d_srcpswd[$i]=${detail[$k]};  ((k=k+1))
                d_sdir[$i]=${detail[$k]};     ((k=k+1))
                d_scurcol[$i]=${detail[$k]};  ((k=k+1))
				d_scurdlvr[$i]=${detail[$k]}; ((k=k+1))
                d_stz[$i]=${detail[$k]};      ((k=k+1))
				d_lbstype[$i]=CP;
                d_mtopts[$i]=CLBSCP;
                ;;
            clbsup )
                d_market[$i]=${detail[$k]};   ((k=k+1))
                d_tdir[$i]=${detail[$k]};     ((k=k+1))
                d_srcip[$i]=${detail[$k]};    ((k=k+1))
                d_srcid[$i]=${detail[$k]};    ((k=k+1))
                d_srcuser[$i]=${detail[$k]};  ((k=k+1))
                d_srcpswd[$i]=${detail[$k]};  ((k=k+1))
                d_sdir[$i]=${detail[$k]};     ((k=k+1))
                d_scurcol[$i]=${detail[$k]};  ((k=k+1))
				d_scurdlvr[$i]=${detail[$k]}; ((k=k+1))
                d_stz[$i]=${detail[$k]};      ((k=k+1))
				d_lbstype[$i]=UP;
                d_mtopts[$i]=CLBSUP;
                ;;

            * ) echo "{$cmd}: ${openclbs_type[$j]} is not valid Openwave CLBS type"
                exit 1
                ;;
        esac

        ((i=i+1))
    done

    ((j=j+1))
done
}

################################################################################
#                                                                              #
#                                Main Script                                   #
#                                                                              #
################################################################################

cmd=`basename $0`

test $# != 1 && usage

# Check the environment variable
if ([ -z $FLEXDAHOME ] || [ ! -d $FLEXDAHOME ]); then
    echo "${cmd}: FLEXDAHOME directory not found"
    exit 1
fi

cfgfile=$1

# cfg directory
CFGDIR=$FLEXDAHOME/cfg

# log directory
LOGDIR=$FLEXDAHOME/log

# data directory
DATADIR=$FLEXDAHOME/data

# backup directory
dtfmt=`date +$(echo :Y:m:d:H:M | tr ":" "%")`
SAVEDIR=$FLEXDAHOME/cfg/backup/openclbs_$dtfmt


#  Each Openwave CLBS process common info
# CLBSPROCS    VDR    DSVR NETYPE DTYPE    PRD(M) ENTRY RETRY CRONTM           DIR            MON?
# ---------    ---    ---- ------ -----    ------ ----- ----- ------           -----          ----
set -A  CLBSPROCS   \
    CLBSCP     open   ""   clbscp clbscp   30     144   3     "15,45"          "src|dist"     N   \
    CLBSUP     open   ""   clbsup clbsup   30     144   3     "15,45"          "src|dist"     N   \


#  Number of CLBSPROCS attributes
NA_CLBSPROCS=11


# Initialize local variable
unset d_market d_tdir d_srcip d_srcid d_srcuser d_srcpswd
unset d_sdir d_scurcol d_scurdlvr d_stz
unset d_lbstype
unset d_mtopts d_monopt


# Validate the setup config file and populate array
check_cfg_file $cfgfile

# Open a file descriptor for the cron file
cron_file=$CFGDIR/${HOST}.openclbs.cron

fd_cron=4
create_file $fd_cron $cron_file


# Loop through each type and make the setup

ans=0
j=0
while (($j < ${#d_market[*]})); do

    p=0
    unset mt_opt
    set -A mt_opt $(echo ${d_mtopts[$j]} | tr "," " ")
    while (($p < ${#mt_opt[*]})); do
        # Get common information for each openclbs option
        unset c_optname c_vendor c_datasrv c_netype c_dtype
        unset c_period c_entries c_retries c_crontm c_ddir c_monproc
        unset found nename
        unset m_procname m_progname m_ldir m_ddir mlogdir mdatadir
        unset tlogdir tdatadir

        # Validate option type
        found=0
        x=0
        while (($x < ${#CLBSPROCS[*]})); do
            if [ "${mt_opt[$p]}" == "${CLBSPROCS[$x]}" ]; then
                found=1;
                break;
            else
                ((x=x+$NA_CLBSPROCS))
            fi
        done

        if [ $found == 0 ]; then
            echo "${cmd}: ${mt_opt[$p]} is not a supported openclbs option type"
            ((p=p+1))
            continue
        fi

        # Get the CLBSPROCS common info
        c_optname=${CLBSPROCS[$x]}; ((x=x+1))
        c_vendor=${CLBSPROCS[$x]};  ((x=x+1))
        c_datasrv=${CLBSPROCS[$x]}; ((x=x+1))
        c_netype=${CLBSPROCS[$x]};  ((x=x+1))
        c_dtype=${CLBSPROCS[$x]};   ((x=x+1))
        c_period=${CLBSPROCS[$x]};  ((x=x+1))
        c_entries=${CLBSPROCS[$x]}; ((x=x+1))
        c_retries=${CLBSPROCS[$x]}; ((x=x+1))
        set -A c_crontm $(echo ${CLBSPROCS[$x]} | tr "|" " "); ((x=x+1))
        c_ddir=${CLBSPROCS[$x]};  ((x=x+1))
        typeset -u c_monproc=${CLBSPROCS[$x]}; ((x=x+1))

        # Create FlexDA monitor log and data directories
        if [ "${c_monproc}" == "Y" ]; then
            n=0
            set -A d_monopt "${d_monopt} $c_optname"
            set -A mon_detail $(eval echo \$\{$c_optname\_mon[*]})

            if ([ ${#mon_detail[*]} == 0 ] || \
                [ `expr ${#mon_detail[*]} % 4` != 0 ]); then
                echo "${cmd}: ${c_optname}_mon not specified properly."
                exit 1
            fi

            i=0
            while (($n < ${#mon_detail[*]})); do
                m_procname=${mon_detail[$n]}; ((n=n+1))
                m_progname=${mon_detail[$n]}; ((n=n+1))
                m_ldir=${mon_detail[$n]};     ((n=n+1))
                m_ddir=${mon_detail[$n]};     ((n=n+1))

                ((i=i+1))
            done
        fi

        nename=${d_srcid[$j]}
        jobname=${d_market[$j]}.${nename}.${c_optname}
        if [ "${c_datasrv}" != "" ]; then
            datasrv_field="_${c_datasrv}${d_datasrvid[$j]}"
        else
            datasrv_field=""
        fi

        edir=${d_market[$j]}
        tdir=${c_optname}

        #
        # Make required directories
        #

        tlogdir=${LOGDIR}/${edir}
        tdatadir=${DATADIR}/${edir}/${d_srcid[$j]}/${tdir}

        for dir in $(echo $c_ddir | tr "|" " "); do
            set -A mdatadir "${mdatadir[*]} ${dir}"
        done

        #
        # Append monitor log and data directory
        #

        if [ -n "${mdatadir[*]}" ]; then
            unset ddir
            for dir in ${mdatadir[*]}; do
                ddir="$ddir $tdatadir/$dir"
            done

            tdatadir="$tdatadir $ddir"
        fi

        for dir in $tlogdir $tdatadir; do
            if [ ! -d $dir ]; then
                echo "${cmd}: making directory $dir"
                mkdir -p $dir
            fi
        done


        #
        # Creates .chk and .icf file
        #

        chkfname=$jobname.chk
        icffname=$jobname.icf
        cfgfname=$jobname.cfg

	if [ ! -f $LOGDIR/$chkfname ]
    then
        ans=1
    else
        ans=0
    fi

        if [ ans -eq 1 ]; then
	    	create_file 0 $LOGDIR/$chkfname
            icf_create -n $c_entries -r $c_retries \
                $c_period $LOGDIR/$chkfname
        fi


	    fd=5
		create_file $fd $CFGDIR/$icffname

	    # Prints ICF header info
            typeset -u title="$c_vendor $c_dtype Data Transfer"
            print_common_icf $fd "$title" "${d_market[$j]}" "$c_optname" \
                "${d_srcid[$j]}" "$c_dtype"

            case  $c_optname in
                CLBSCP|CLBSUP )
                    print_clbs_icf $fd ${d_tdir[$j]} ${d_srcip[$j]} ${d_srcuser[$j]} \
                        ${d_srcpswd[$j]} ${d_srcid[$j]} ${d_sdir[$j]} \
                        ${d_scurcol[$j]} ${d_scurdlvr[$j]} ${d_lbstype[$j]} \
                        ${lhost[0]} ${d_stz[$j]}
                    ;; 

                * ) echo "${cmd}: $c_optname not supported";;
            esac


             #
             # Prints Prospect target host info
             #

            print_thost_icf $fd ${thost[1]} ${thost[2]} ${thost[3]} ${d_tdir[$j]}


         #
         # Add cron entry in crontab file
         #

        mi=${c_crontm[0]:-*}
        hr=${c_crontm[1]:-*}
        dm=${c_crontm[2]:-*}
        my=${c_crontm[3]:-*}
        dw=${c_crontm[4]:-*}

        tlogdir=`echo $tlogdir|sed "s/ .*$//"`

        if [ $fd_cron ] && [ -n $fd_cron ] ]; then
            print -u$fd_cron "$mi $hr $dm $my $dw ksh -c \". $FLEXDAHOME/.kshrc; icf_chat.exp -i $icffname -L $tlogdir,${d_market[$j]}.${d_srcid[$j]}.$c_optname.,.log,1440,40000,7\""
        fi

        ((p=p+1))
    done
    ((j=j+1))
done


# Add FlexDA system health check jobs

if ([ $fd_cron ] && [ -n $fd_cron ]); then
    print -u$fd_cron \
        "0,30 * * * * ksh -c \". $FLEXDAHOME/.kshrc; check_error.pl\""
    print -u$fd_cron \
        "1,31 * * * * ksh -c \". $FLEXDAHOME/.kshrc; page_error.pl\""
fi

cd $CFGDIR
cat ${HOST}.*.cron > ${HOST}.cron

#
# Remove the duplicated FlexDA system health check jobs
# Note: Append after all vendor specific cronjobs.
#

STR_CHECK_ERROR="check_error\.pl"
STR_PAGE_ERROR="page_error\.pl"

grep -v ${STR_CHECK_ERROR} ${HOST}.cron | grep -v ${STR_PAGE_ERROR} > ${HOST}.cron.$$
mv ${HOST}.cron.$$ ${HOST}.cron

#
# Add FlexDA system health check jobs at the end of cronjobs,
# outside all vendor-specific cronjob blocks
#

echo "0,30 * * * * ksh -c \". $FLEXDAHOME/.kshrc; check_error.pl\"" >> ${HOST}.cron
echo "1,31 * * * * ksh -c \". $FLEXDAHOME/.kshrc; page_error.pl\"" >> ${HOST}.cron

if [ -d $SAVEDIR ]
then
	echo "${cmd}: Files backup in ${SAVEDIR}"
fi

echo "${cmd}: Done!"
